/**********************************

libcachewriter public interface

**********************************/

#ifndef CACHEWRITER_H
#define CACHEWRITER_H

#include <libmemcached/memcached.h>
#include <libwebguard-utils/wg_types.h>

#include <libwebguard-utils/str_t.h>

#ifdef __cplusplus
extern "C"{
#endif

typedef struct cache_writer *HCW;

typedef struct cw_config
{
	wg_memcached_peer       memc_acl;               /* [memcached_acl] section in .ini file */
	wg_range                username_len_range;     /* [cache_limits] section in .ini file */
} cw_config;

typedef enum
{
	JSON_ACL_COMMON,
	JSON_ACL_URL,
} cw_data_t;


int cw_load_config(const char* filename,    // __in имя файла конфигурации
                    cw_config* conf);       // __out загруженная конфигурация

void cw_clear_config( cw_config* conf );    // __in конфигурация

HCW cw_create(cw_config* conf);             // __in конфигурация для создания cache_writer


void cw_free(HCW hcw);                      // __in хэндл созданного cache_writer


int cw_write(HCW hcw,                       // __in хэндл созданного cache_writer
            const str_t* key_prefix,        // __in префикс ключа
            const char* data,               // __in данные для записи в кэш
            cw_data_t data_type);           // __in тип данных


int cw_flush_cache(HCW hcw);                // __in хэндл созданного cache_writer

#ifdef __cplusplus
}
#endif

#endif /* CACHEWRITER_H */
