/*********************************************
Simple stderr logger based on variadic macros
*********************************************/

#ifndef LOGGER_H
#define LOGGER_H

#include <stdio.h>

/* Logger configuration */

#define LOG_ERROR_ENABLE
#define LOG_WARN_ENABLE
#define LOG_INFO_ENABLE

#if defined(DEBUG) || defined(_DEBUG)
#	define LOG_DEBUG_ENABLE
#	define LOG_TRACE_ENABLE
#endif

/* ******************** */


#define LOG_TRAILER "\n"

#if defined(LOG_ERROR_ENABLE)

#if defined(DEBUG) || defined(_DEBUG)
#   define LOG_ERROR(format, ...)			fprintf(stderr, "ERROR: " "%s(%u): " format LOG_TRAILER, __FILE__, __LINE__, ##__VA_ARGS__)
#else
#	define LOG_ERROR(format, ...)			fprintf(stderr, "ERROR: " format LOG_TRAILER, ##__VA_ARGS__)
#endif /* DEBUG || _DEBUG*/

#else
#   define LOG_ERROR(format, ...)
#endif /*LOG_ERROR_ENABLE*/



#if defined(LOG_WARN_ENABLE)

#if defined(DEBUG) || defined(_DEBUG)
#   define LOG_WARN(format, ...)            fprintf(stderr, "WARN: " "%s(%u): " format LOG_TRAILER, __FILE__, __LINE__, ##__VA_ARGS__)
#else
#   define LOG_WARN(format, ...)			fprintf(stderr, "WARN: " format LOG_TRAILER, ##__VA_ARGS__)
#endif /* DEBUG || _DEBUG*/

#else
#   define LOG_WARN(format, ...)
#endif /*LOG_WARN_ENABLE*/



#if defined(LOG_INFO_ENABLE)

#if defined(DEBUG) || defined(_DEBUG)
#   define LOG_INFO(format, ...)            fprintf(stderr, "INFO: " "%s(%u): " format LOG_TRAILER, __FILE__, __LINE__, ##__VA_ARGS__)
#else
#   define LOG_INFO(format, ...)			fprintf(stderr, "INFO: " format LOG_TRAILER, ##__VA_ARGS__)
#endif /* DEBUG || _DEBUG*/

#else
#   define LOG_INFO(format, ...)
#endif /*LOG_INFO_ENABLE*/



#if defined(LOG_DEBUG_ENABLE)

#if defined(DEBUG) || defined(_DEBUG)
#	define LOG_DEBUG(format, ...)           fprintf(stderr, "DEBUG: " "%s(%u): " format LOG_TRAILER, __FILE__, __LINE__, ##__VA_ARGS__)
#else
#   define LOG_DEBUG(format, ...)			fprintf(stderr, "DEBUG: " format LOG_TRAILER, ##__VA_ARGS__)
#endif /* DEBUG || _DEBUG*/

#else
#	define LOG_DEBUG(format, ...)
#endif /*LOG_DEBUG_ENABLE*/



#if defined(LOG_TRACE_ENABLE)

#if defined(DEBUG) || defined(_DEBUG)
#	define LOG_TRACE(format, ...)           fprintf(stderr, "TRACE: " "%s(%u): " format LOG_TRAILER, __FILE__, __LINE__, ##__VA_ARGS__)
#else
#   define LOG_TRACE(format, ...)			fprintf(stderr, "TRACE: " format LOG_TRAILER, ##__VA_ARGS__)
#endif /* DEBUG || _DEBUG*/

#else
#	define LOG_TRACE(format, ...)
#endif /*LOG_TRACE_ENABLE*/


#endif /*LOGGER_H*/
