#ifndef URLAUTHDATA_H
#define URLAUTHDATA_H

#include <libwebguard-utils/str_t.h>
#include <libwebguard-utils/wg_limits.h>

#define INFINITE_LIFETIME       0
#define INFINITE_NUMBER         -1

typedef struct client_identity_s
{
    str_t ip_address;
    str_t sessid;

} client_identity_t;

typedef struct user_identity_s
{
    str_t module_id;
    str_t login;

} user_identity_t;

typedef struct user_credentials
{
    str_t user_id;
    str_t password;

} user_credentials;

typedef struct request_param_t
{
    char* name;
    char* value;
    struct request_param_t* _next;

} request_param_t;

typedef struct url_request_t
{
    str_t method;
    /* поле обязательно является null-terminated строкой
     * ( этого требует работа с регулярными выражениями ) и необходимостью выполнять unenscape */
    str_t path;
    request_param_t* params;
    request_param_t* post_params;
} url_request_t;

typedef struct wg_request_ctx_s
{
//    wg_flag_t client_identified : 1;
//    wg_flag_t user_identified : 1;

    client_identity_t client_id;
    user_identity_t user_id;


} wg_request_ctx_t;

typedef enum auth_status_t
{
    AUTH_ERROR                  = -1,
    AUTH_FAIL                   = 0,
    AUTH_SUCCESS                = 1,

    AUTH_DENIED                 = 2,
    AUTH_GRANTED                = 3,

    AUTH_CLIENT_NOT_IDENTIFIED  = 4,
    AUTH_USER_NOT_IDENTIFIED    = 5,

    AUTH_TEMPORARILY_BLOCKED    = 6,
    AUTH_AUTHENTICATION_FAILED  = 7,
    AUTH_PERMANENTLY_BLOCKED    = 8,
    AUTH_PASS_EXPIRED           = 9,
    AUTH_PASS_NOTIFY_CHANGE     = 10, /// TODO! AUTH_LOGGED_IN + flag
    AUTH_TOO_MANY_SESSIONS      = 11,
    AUTH_LOGGED_IN              = 12,
    AUTH_LOGGED_OUT             = 13

} auth_status_t;

/** TODO! add module_id field before username (login) */
typedef struct
{
    const str_t         remote_host;
    const str_t         sid;
    const str_t         username;
    const str_t         method;
    const str_t         path;
    const str_t         func_name;
    auth_status_t       status;
    const str_t         additional_info;

} audit_info;

#endif /* URLAUTHDATA_H */
